﻿using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding.Views;
using HeuristicLab.Visualization;

namespace HeuristicLab.Problems.DataAnalysis.Symbolic.Views {
  public partial class SymbolicDataAnalysisExpressionLineageExplorerChart : ChartControl {
    public SymbolicDataAnalysisExpressionLineageExplorerChart() {
      InitializeComponent();
      if (Chart == null) { Chart = new Chart(0, 0, PreferredSize.Width, PreferredSize.Height); }
    }

    public void Clear() {
      Chart.Group.Clear();
    }

    public void Add(IPrimitive primitive) {
      Chart.Group.Add(primitive);
    }

    public void AddRange(IEnumerable<IPrimitive> primitives) {
      Chart.Group.AddRange(primitives);
    }

    public void Draw(Graphics graphics) {
      Chart.Group.Draw(graphics);
    }

    public bool UpdateEnabled {
      get { return Chart.UpdateEnabled; }
      set { Chart.UpdateEnabled = value; }
    }

    public void EnforceUpdate() {
      Chart.EnforceUpdate();
    }

    public void FlipVertical() {
      foreach (var primitive in Chart.Group.Primitives) {
        var tile = primitive as SymbolicExpressionTreeTile;
        if (tile != null) {
          foreach (var p in tile.Primitives) {
            FlipPrimitive(p);
          }
        } else {
          FlipPrimitive(primitive);
        }
      }
    }

    public void FlipPrimitive(IPrimitive primitive) {
      var height = pictureBox.Height;
      var rect = primitive as RectangularPrimitiveBase;
      var line = primitive as LinearPrimitiveBase;
      if (rect != null) {
        rect.SetPosition(rect.LowerLeft.X, height - rect.UpperRight.Y, rect.UpperRight.X, height - rect.LowerLeft.Y);
      } else if (line != null) {
        line.SetPosition(line.Start.X, height - line.Start.Y, line.End.X, height - line.End.Y);
      }
    }

    protected override void pictureBox_MouseUp(object sender, MouseEventArgs e) {
      base.pictureBox_MouseUp(sender, e);
      // get the selected primitive and update it's graphics
      var point = new Point(e.X, e.Y);

      var selectedPrimitives = Chart.GetAllPrimitives(point);
      // first primitive in the list should be the tile, the second one should be the actual primitive representing a tree node
      var tile = (SymbolicExpressionTreeTile)selectedPrimitives[0];
      var selectedPrimitive = (RectangularPrimitiveBase)selectedPrimitives[1];
      var tree = tile.SymbolicExpressionTree; // use the tree to get the genealogy graph node and the fragments used for tracing building blocks

      var pen = new Pen(Color.Blue);
      using (var g = Graphics.FromImage(pictureBox.Image)) {
        g.DrawRectangle(pen, (float)selectedPrimitive.LowerLeft.X, (float)selectedPrimitive.LowerLeft.Y, (float)selectedPrimitive.Size.Width, (float)selectedPrimitive.Size.Height);
      }
    }
  }
}
